/*
 * Copyright (c) 1995 Cartoactive Systems
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the names of the copyright holders not be used in
 * advertising or publicity pertaining to distribution of the software
 * without specific, written prior permission.  The copyright holders make
 * no representations about the suitability of this software for any purpose.
 * It is provided "as is" without express or implied warranty.
 *
 * THE COPYRIGHT HOLDERS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.
 * IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
 * FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
 * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 * 
 */

#include <Unidraw/catalog.h>
#include <MapUnidraw/gscmd.h>
#include <MapUnidraw/gseditor.h>
#include <MapUnidraw/gsview.h>
#include <MapUnidraw/mapeditor.h>
#include <IVGlyph/fieldedit.h>
#include <IVGlyph/figure.h>

#include <InterViews/action.h>
#include <InterViews/background.h>
#include <InterViews/display.h>
#include <InterViews/label.h>
#include <InterViews/layout.h>
#include <InterViews/session.h>
#include <InterViews/style.h>
#include <IV-look/dialogs.h>
#include <IV-look/kit.h>
#include <IV-X11/xdisplay.h>
#include <IV-X11/xcolor.h>
#include <IV-X11/xpattern.h>
#undef None

#include <OS/memory.h>
#include <OS/string.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <stream.h>
#include <strstream.h>

declareActionCallback(GSBrushCmd)
implementActionCallback(GSBrushCmd)
declareActionCallback(GSColorCmd)
implementActionCallback(GSColorCmd)
declareActionCallback(GSPatternCmd)
implementActionCallback(GSPatternCmd)
declareActionCallback(AddThemeCmd)
implementActionCallback(AddThemeCmd)

GSEditor::GSEditor(Catalog* cat, ObservableGS* ogs, MapEditor* me)
: MonoGlyph()
{
    gs = ogs;
    catalog = cat;
    mapeditor = me;
    init_body();
}

declareActionCallback(Session)

void GSEditor::init_body() {
    WidgetKit& wk = *WidgetKit::instance();
    LayoutKit& lk = *LayoutKit::instance();
    DialogKit& dk = *DialogKit::instance();
    PolyGlyph* mainglyph = lk.vbox();
    BrushView* brushview = new BrushView(gs);
    PatternView* patternview = new PatternView(gs);
    Menu * top_menu = wk.menubar();

    MenuItem* thememi = wk.menubar_item("Theme");
    thememi->menu(wk.pulldown());
    AddThemeCmd* atcmd = new AddThemeCmd(mapeditor);
    MenuItem* item = wk.menu_item(wk.label("Add Theme"));
    thememi->menu()->append_item(item);
    item->action(new ActionCallback(AddThemeCmd)(atcmd, &AddThemeCmd::Execute));
    top_menu->append_item(thememi);

    MenuItem* brushmi = wk.menubar_item("Brush");
    brushmi->menu(wk.pulldown());
    int i = 1;
    PSBrush* br = catalog->ReadBrush("brush", i);
    while (br != nil) {
	GSBrushCmd* cmd = new GSBrushCmd(this, br);
	MenuItem* item = wk.menu_item(MenuLine(br));
	brushmi->menu()->append_item(item);
	item->action(new ActionCallback(GSBrushCmd)(cmd, &GSBrushCmd::Execute));
	br = catalog->ReadBrush("brush", ++i);
    }
    top_menu->append_item(brushmi);

    MenuItem* patmi = wk.menubar_item("Pattern");
    patmi->menu(wk.pulldown());
    i = 1;
    PSPattern* pat = catalog->ReadPattern("pattern", i);
    while (pat != nil) {
	GSPatternCmd* cmd = new GSPatternCmd(this, pat);
	MenuItem* item = wk.menu_item(MenuPatRect(pat));
	patmi->menu()->append_item(item);
	item->action(new ActionCallback(GSPatternCmd)(cmd, &GSPatternCmd::Execute));
	pat = catalog->ReadPattern("pattern", ++i);
    }
    top_menu->append_item(patmi);

    MenuItem* fgcolmi = wk.menubar_item("FgColor");
    fgcolmi->menu(wk.pulldown());
    i = 1;
    PSColor* fgcol = catalog->ReadColor("fgcolor", i);
    while (fgcol != nil) {
	GSColorCmd* cmd = new GSColorCmd(this, fgcol, nil);
	MenuItem* item = wk.menu_item(MenuRect(fgcol));
	fgcolmi->menu()->append_item(item);
	item->action(new ActionCallback(GSColorCmd)(cmd, &GSColorCmd::Execute));
	fgcol = catalog->ReadColor("fgcolor", ++i);
    }
    top_menu->append_item(fgcolmi);

    MenuItem* bgcolmi = wk.menubar_item("BgColor");
    bgcolmi->menu(wk.pulldown());
    i = 1;
    PSColor* bgcol = catalog->ReadColor("bgcolor", i);
    while (bgcol != nil) {
	GSColorCmd* cmd = new GSColorCmd(this, nil, bgcol);
	MenuItem* item = wk.menu_item(MenuRect(bgcol));
	bgcolmi->menu()->append_item(item);
	item->action(new ActionCallback(GSColorCmd)(cmd, &GSColorCmd::Execute));
	bgcol = catalog->ReadColor("bgcolor", ++i);
    }
    top_menu->append_item(bgcolmi);

    Glyph* desc = lk.hfixed(
	descfe = dk.field_editor("", wk.style()),
	150
    );
    Glyph* expr = lk.hfixed(
	exprfe = dk.field_editor("", wk.style()),
	350
    );
    Glyph* tgs = lk.hbox(
	lk.vcenter(brushview),
	lk.hspace(4),
	lk.vcenter(patternview)
    );
    mainglyph->append(
	lk.vbox(
	    lk.hcenter(top_menu),
	    lk.hcenter(
		wk.outset_frame(
		    lk.vnatural(
			lk.hbox(
			    lk.vcenter(
				lk.margin(
				    lk.vbox(
					lk.overlay(
					    lk.shape_of(desc),
					    wk.label("Description:")
					),
					lk.vspace(10),
					desc
				    ),
				    10
				)
			    ),
			    lk.vcenter(
				lk.margin(
				    lk.vbox(
					lk.overlay(
					    lk.shape_of(expr),
					    wk.label("Expression:")
					),
					lk.vspace(10),
					expr
				    ),
				    10
				)
			    ),
			    lk.vcenter(
				lk.margin(
				    lk.vbox(
					lk.overlay(
					    lk.shape_of(tgs),
					    wk.label("Graphic State:")
					),
					lk.vspace(5),
					tgs
				    ),
				    10
				)
			    )
			),
			100
		    )
		)
	    )
	)
    );
    body(mainglyph);
}

static float MENU_WIDTH = 1.3;   /* in cm */
static float MENU_HEIGHT = 0.5;

Glyph* GSEditor::MenuLine(PSBrush* br) {
    LayoutKit& lk = *LayoutKit::instance();
    WidgetKit& wk = *WidgetKit::instance();
    if (br->None())
	return lk.hbox(lk.hglue(), wk.label("None"), lk.hglue());
    else
	return lk.margin(new Fig31Line(br, wk.foreground(), nil,
				       0, 0, MENU_WIDTH*ivcm, 0),
			 0.1*MENU_WIDTH*ivcm, 0.4*MENU_HEIGHT*ivcm);
}

Glyph* GSEditor::MenuPatRect (PSPattern * pat) {
    LayoutKit& lk = *LayoutKit::instance();
    WidgetKit& wk = *WidgetKit::instance();
    Brush * brush = new Brush(0.0);
    Resource::ref(brush);
    Color * color = new Color(*(wk.foreground()));
    Resource::ref(color);
    color->rep(Session::instance()->default_display()->rep()->default_visual_)->stipple_ = pat->rep()->pixmap_;
    Coord w = (MENU_WIDTH*ivcm);
    Coord h = (MENU_HEIGHT*ivcm);
    
    if (pat->None()) {
	return lk.fixed(lk.vbox(lk.vglue(),
				lk.hbox(lk.hglue(), wk.label("None"), lk.hglue()),
				lk.vglue()),
			1.4*MENU_WIDTH*ivcm,
			1.4*MENU_HEIGHT*ivcm);
    } else {
	return lk.margin(new Fig31Rectangle(brush, wk.foreground(), color,
					    0, 0, w, h),
			 0.2*MENU_WIDTH*ivcm,
			 0.2*MENU_HEIGHT*ivcm);
    }
}

Glyph* GSEditor::MenuRect (Color * color) {
    Brush * brush = new Brush(0.0);
    Coord w = (MENU_WIDTH*ivcm);
    Coord h = (MENU_HEIGHT*ivcm);
    
    Resource::ref(brush);
    return new Fig31Rectangle(brush, WidgetKit::instance()->foreground(),
			      color, 0, 0, w, h);
}
